@@ -1,4 +1,4 @@
-from typing import Tuple, Union
+from typing import List, Tuple, Union
 
 import numpy as np
 from mne import BaseEpochs
@@ -8,7 +8,7 @@ from mne.utils import warn
 from numpy.typing import NDArray
 from scipy.signal import resample_poly
 
-from ..utils import _validate_inst_and_ica
+from ..utils._checks import _validate_inst_and_ica
 from .utils import _gdatav4, _mne_to_eeglab_locs, _next_power_of_2, _pol2cart
 
 
@@ -24,11 +24,11 @@ def get_iclabel_features(inst: Union[BaseRaw, BaseEpochs], ica: ICA):
 
     Returns
     -------
-    topo : np.ndarray of shape (32, 32, 1, n_components)
+    topo : array of shape (32, 32, 1, n_components)
         The topoplot feature.
-    psd : np.ndarray of shape (1, 100, 1, n_components)
+    psd : array of shape (1, 100, 1, n_components)
         The psd feature.
-    autocorr : np.ndarray of shape (1, 100, 1, n_components)
+    autocorr : array of shape (1, 100, 1, n_components)
         The autocorrelations feature. Depending on the length of the
         raw data passed in, different methods of computing autocorrelation
         will be used. See :footcite:`iclabel2019` for details.
@@ -45,7 +45,7 @@ def get_iclabel_features(inst: Union[BaseRaw, BaseEpochs], ica: ICA):
     if inst.info["custom_ref_applied"] == 0:
         warn(
             f"The provided {'Raw' if isinstance(inst, BaseRaw) else 'Epochs'} instance "
-            "does not seems to be referenced to a common average reference (CAR). "
+            "does not seem to be referenced to a common average reference (CAR). "
             "ICLabel was designed to classify features extracted from an EEG dataset "
             "referenced to a CAR (see the 'set_eeg_reference()' method for Raw and "
             "Epochs instances)."
@@ -58,12 +58,25 @@ def get_iclabel_features(inst: Union[BaseRaw, BaseEpochs], ica: ICA):
             "bandpass filtered between 1 and 100 Hz (see the 'filter()' method for Raw "
             "and Epochs instances)."
         )
+    # confirm that the ICA uses an infomax extended
+    method_ = ica.method not in ("infomax", "picard")
+    extended_ = "extended" not in ica.fit_params or ica.fit_params["extended"] is False
+    ortho_ = "ortho" not in ica.fit_params or ica.fit_params["ortho"] is True
+    ortho_ = ortho_ if ica.method == "picard" else False
+    if any((method_, extended_, ortho_)):
+        warn(
+            f"The provided ICA instance was fitted with a '{ica.method}' algorithm. "
+            "ICLabel was designed with extended infomax ICA decompositions. To use the "
+            "extended infomax algorithm, use the 'mne.preprocessing.ICA' instance with the "
+            "arguments 'ICA(method='infomax', fit_params=dict(extended=True))' (scikit-learn) or "
+            "'ICA(method='picard', fit_params=dict(ortho=False, extended=True))' (python-picard)."
+        )
 
     icawinv, _ = _retrieve_eeglab_icawinv(ica)
     icaact = _compute_ica_activations(inst, ica)
 
     # compute topographic feature (float32)
-    topo = _eeg_topoplot(inst, icawinv)
+    topo = _eeg_topoplot(inst, icawinv, ica.ch_names)
 
     # compute psd feature (float32)
     psd = _eeg_rpsd(inst, ica, icaact)
@@ -89,7 +102,7 @@ def _retrieve_eeglab_icawinv(
     ica: ICA,
 ) -> Tuple[NDArray[float], NDArray[float]]:
     """
-    Retrieves 'icawinv' from an MNE ICA instance.
+    Retrieve 'icawinv' from an MNE ICA instance.
 
     Parameters
     ----------
@@ -131,7 +144,7 @@ def _compute_ica_activations(inst: Union[BaseRaw, BaseEpochs], ica: ICA) -> NDAr
 
     Notes
     -----
-    In EEGLAB, the ICA activation are computed after the data and the ICA
+    In EEGLAB, the ICA activations are computed after the data and the ICA
     decomposition are re-referenced to a common average, if the field 'EEG.ref'
     is different from 'averef'. The EEGLAB sample dataset's field 'EEG.ref' is
     set to 'common', thus triggering the re-referencing with 'pop_reref' which
@@ -155,12 +168,13 @@ def _compute_ica_activations(inst: Union[BaseRaw, BaseEpochs], ica: ICA) -> NDAr
 
 
 # ----------------------------------------------------------------------------
-def _eeg_topoplot(inst: Union[BaseRaw, BaseEpochs], icawinv: NDArray[float]) -> NDArray[float]:
+def _eeg_topoplot(
+    inst: Union[BaseRaw, BaseEpochs], icawinv: NDArray[float], picks: List[str]
+) -> NDArray[float]:
     """Topoplot feature."""
-    # TODO: Selection of channels is missing.
     ncomp = icawinv.shape[-1]
     topo = np.zeros((32, 32, 1, ncomp))
-    rd, th = _mne_to_eeglab_locs(inst)
+    rd, th = _mne_to_eeglab_locs(inst, picks)
     th = np.pi / 180 * th  # convert degrees to radians
     for it in range(ncomp):
         temp_topo = _topoplotFast(icawinv[:, it], rd, th)
@@ -170,8 +184,7 @@ def _eeg_topoplot(inst: Union[BaseRaw, BaseEpochs], icawinv: NDArray[float]) ->
 
 
 def _topoplotFast(values: NDArray[float], rd: NDArray[float], th: NDArray[float]) -> NDArray[float]:
-    """Implements topoplotFast.m from MATLAB. Each topographic map is a 32x32
-    images."""
+    """Implement topoplotFast.m from MATLAB. Each topographic map is a 32x32 images."""
     # constants
     GRID_SCALE = 32  # number of pixels
     rmax = 0.5  # actual head radius
@@ -239,8 +252,7 @@ def _eeg_rpsd_constants(
     inst: Union[BaseRaw, BaseEpochs],
     ica: ICA,
 ) -> Tuple[int, int, int, int, NDArray[int], NDArray[float], NDArray[int]]:
-    """Computes the constants before ``randperm`` is used to compute the
-    subset."""
+    """Compute the constants before ``randperm`` is used to compute the subset."""
     # in MATLAB, 'pct_data' variable is never provided and is always initialized
     # to 100. 'pct_data' is only used in a division by 100.. and thus has no
     # impact and is omitted here.
@@ -300,11 +312,11 @@ def _eeg_rpsd_compute_psdmed(
         if isinstance(inst, BaseRaw):
             temp = np.hstack([icaact[it, index[:, k]] for k in range(index.shape[-1])])
             temp = temp.reshape(*index.shape, order="F")
+            # equivalent to:
+            # np.vstack([icaact[it, index[:, k]] for k in range(index.shape[-1])]).T
         elif isinstance(inst, BaseEpochs):
-            temp = np.hstack([icaact[it, index[:, k], :] for k in range(index.shape[-1])])
-            temp = temp.reshape(index.shape[0], len(inst), order="F")
-        else:
-            raise RuntimeError  # should never happen
+            temp = np.vstack([icaact[it, index[:, k], :] for k in range(index.shape[-1])])
+            temp = temp.reshape(index.shape[0], index.shape[1] * len(inst), order="F")
         temp = (temp[:, subset].T * window).T
         temp = np.fft.fft(temp, n_points, axis=0)
         temp = temp * np.conjugate(temp)
@@ -319,8 +331,7 @@ def _eeg_rpsd_compute_psdmed(
 def _eeg_rpsd_format(
     psd: NDArray[float],
 ) -> NDArray[float]:
-    """Apply the formatting steps after 'eeg_rpsd.m' from the MATLAB feature
-    extraction."""
+    """Apply the formatting steps after 'eeg_rpsd.m' from the MATLAB feature extraction."""
     # extrapolate or prune as needed
     nfreq = psd.shape[1]
     if nfreq < 100:
@@ -355,9 +366,10 @@ def _eeg_rpsd_format(
 
 
 def _eeg_autocorr_welch(raw: BaseRaw, ica: ICA, icaact: NDArray[float]) -> NDArray[float]:
-    """Autocorrelation feature applied on raw object with at least 5 * fs
-    samples (5 seconds).
-    MATLAB: 'eeg_autocorr_welch.m'."""
+    """Autocorrelation feature applied on raw object with at least 5 * fs samples (5 seconds).
+
+    MATLAB: 'eeg_autocorr_welch.m'.
+    """
     assert isinstance(raw, BaseRaw)  # sanity-check
 
     # in MATLAB, 'pct_data' variable is never provided and is always initialized
@@ -431,9 +443,10 @@ def _eeg_autocorr_welch(raw: BaseRaw, ica: ICA, icaact: NDArray[float]) -> NDArr
 
 
 def _eeg_autocorr(raw: BaseRaw, ica: ICA, icaact: NDArray[float]) -> NDArray[float]:
-    """Autocorrelation feature applied on raw object that do not have enough
-    sampes for eeg_autocorr_welch.
-    MATLAB: 'eeg_autocorr.m'."""
+    """Autocorrelation feature applied on raw object without enough sampes for eeg_autocorr_welch.
+
+    MATLAB: 'eeg_autocorr.m'.
+    """
     assert isinstance(raw, BaseRaw)  # sanity-check
 
     # in MATLAB, 'pct_data' variable is neither provided or used, thus it is
@@ -466,7 +479,9 @@ def _eeg_autocorr(raw: BaseRaw, ica: ICA, icaact: NDArray[float]) -> NDArray[flo
 
 def _eeg_autocorr_fftw(epochs: BaseEpochs, ica: ICA, icaact: NDArray[float]) -> NDArray[float]:
     """Autocorrelation feature applied on epoch object.
-    MATLAB: 'eeg_autocorr_fftw.m'."""
+
+    MATLAB: 'eeg_autocorr_fftw.m'.
+    """
     assert isinstance(epochs, BaseEpochs)  # sanity-check
 
     # in MATLAB, 'pct_data' variable is neither provided or used, thus it is
@@ -496,8 +511,9 @@ def _eeg_autocorr_fftw(epochs: BaseEpochs, ica: ICA, icaact: NDArray[float]) ->
 
 
 def _resample(ac: NDArray[float], fs: Union[int, float]) -> NDArray[float]:
-    """Resample the autocorrelation feature. The comment in EEGLAB is:
+    """Resample the autocorrelation feature.
 
+    The comment in EEGLAB is:
         resample to 1 second at 100 samples/sec
 
     Which translates by: the output array must be of shape (n_comp, 101), thus
@@ -509,7 +525,7 @@ def _resample(ac: NDArray[float], fs: Union[int, float]) -> NDArray[float]:
     Parameters
     ----------
     ac : array
-        Array of shape (n_comp, samples)
+        Array of shape (n_comp, samples).
     fs : int | float
         Sampling frequency of the MNE instance.
     """
